from bs4 import BeautifulSoup
from unittest import TestCase

from ebird.pages.checklists import _get_longitude


class GetLongitudeTests(TestCase):
    """
    Tests for extracting the longitude of the location.

    """

    def setUp(self):
        self.template = """
        <div class="rs-body">
            <h5 class="bd obs-loc">	
                <span class="btn-map">( <a href="http://maps.com/?q=0.0,12.3456789&ll=0.0,%s">Map</a> )</span>
            </h5>
        <div>
        """
        self.value = None

    def parse(self):
        self.assertIsNotNone(self.value)
        content = self.template % self.value
        soup = BeautifulSoup(content, "lxml")
        tag = soup.find('div', class_='rs-body')
        return _get_longitude(tag)

    def test_longitude(self):
        """The longitude is returned as a float."""
        self.value = '12.3456789'
        self.assertEqual(12.3456789, self.parse())

    def test_negative(self):
        """Longitudes can be west of the prime meridian."""
        self.value = '-12.3456789'
        self.assertEqual(-12.3456789, self.parse())

    def test_query_parameter_order(self):
        """The longitude is returned even if the query parameter order changes."""
        self.template = """
        <div class="rs-body">
            <h5 class="bd obs-loc">	
                <span class="btn-map">( <a href="http://maps.com/?ll=0.0,%s&q=0.0,12.3456789">Map</a> )</span>
            </h5>
        <div>
        """
        self.value = '12.3456789'
        self.assertEqual(12.3456789, self.parse())

    def test_precision_changes(self):
        """Longitude may be expressed with any level of precision."""
        self.value = '10.0'
        self.assertEqual(10.0, self.parse())



from bs4 import BeautifulSoup
from unittest import TestCase

from ebird.pages.checklists import _get_count


class GetEntryCountTests(TestCase):
    """
    Tests for extracting the count for each species seen.

    """
    def setUp(self):
        self.template = """
            <tr class="spp-entry">
                <th><h5 class="se-count">%s</h5></th>
            </tr>
        """
        self.value = None

    def parse(self):
        self.assertIsNotNone(self.value)
        content = self.template % self.value
        soup = BeautifulSoup(content, "lxml")
        tag = soup.find('tr', class_='spp-entry')
        return _get_count(tag)

    def test_integer(self):
        """The count for a species is returned as an integer."""
        self.value = '3'
        self.assertEqual(3, self.parse())

    def test_present(self):
        """If species was not counted then None is returned."""
        self.value = 'X'
        self.assertEqual(None, self.parse())

    def test_whitespace(self):
        """Count is returned even if there is surrounding whitespace."""
        self.value = '\n3\n'
        self.assertEqual(3, self.parse())

    def test_present_lower_case(self):
        """The character used to indicate only presence can be lower case."""
        self.value = 'x'
        self.assertEqual(None, self.parse())



import datetime
import json


class JSONDateEncoder(json.JSONEncoder):

    def default(self, o):
        if isinstance(o, datetime.date):
            return o.isoformat()
        if isinstance(o, datetime.time):
            return o.isoformat()
        if isinstance(o, datetime.timedelta):
            return str(o)
        else:
            super().default(o)


def save(fp, values, indent):
    """Save the JSON data to a file or stdout.

    :param fp: the writer.
    :param values: the python data to be saved.
    :param indent: the level of indentation when prettyprinting the output.

    """
    fp.write(json.dumps(values, indent=indent, cls=JSONDateEncoder).encode('utf-8'))
    if fp.name == '<stdout>':
        fp.write(b'\n')



# TODO Add support for entry comments.
# TODO Add support for age/sex breakdown
# TODO Add support for extracting information on uploaded media.
# TODO Add support for Oiled Birds protocol

import datetime
import re

from bs4 import BeautifulSoup

from ebird.pages.utils import get_content

# Base URL for the page to view a checklist - the unique identifier is
# appended.

CHECKLIST_URL = "http://ebird.org/ebird/view/checklist/"

# The list of supported protocols.

PROTOCOLS = (
    'Stationary',
    'Traveling',
    'Incidental',
    'Historical',
    'Area,',
    'Banding',
    'eBird Pelagic Protocol',
    'Nocturnal Flight Call Count',
    'Random',
    'CWC Point Count',
    'CWC Area Count',
    'PROALAS',
    'TNC California Waterbird Count',
    'Rusty BlackbirdSpring Migration Blitz',
    'California Brown Pelican Survey',
)


def get_checklist(identifier):
    """
    Get the data for a checklist from its eBird web page.

    :param identifier: the unique identifier for the checklist.

    :return: a dict containing all the fields extracted from the web page.

    """
    checklist = {}

    content = get_content(CHECKLIST_URL + identifier)
    parser = BeautifulSoup(content, "lxml")

    sections = parser.find_all('div', class_='report-section')

    checklist['identifier'] = identifier
    checklist['location'] = _get_location(sections[0])
    checklist.update(_get_date_and_effort(sections[1]))
    checklist['entries'] = _get_entries(sections[2])
    checklist['complete'] = _get_complete(sections[3])

    return checklist


def _get_location(node):
    """
    Get the detail of the location where the checklist was recorded.

    :param node: the Tag for the body of the Location section.

    :return: a dict containing the fields describing the location.

    """
    checklist = {
        'name': _get_site(node),
        'subnational2': _get_subnational2(node),
        'subnational1': _get_subnational1(node),
        'country': _get_country(node),
        'lat': _get_latitude(node),
        'lon': _get_longitude(node),
    }

    identifier = _get_location_identifier(node)
    if identifier:
        checklist['identifier'] = identifier

    return checklist


def _get_site(node):
    """
    Get the name of the site where the checklist was recorded.

    :param node: the Tag for the body of the Location section.

    :return: the name of the site.

    """
    tag = node.find('h5', class_='obs-loc')
    location = tag.text.split('( Map )')[0]
    site = location.rsplit(',', maxsplit=3)[0]
    return ' '.join(site.split())


def _get_subnational2(node):
    """
    Get the county where the checklist was recorded.

    :param node: the Tag for the body of the Location section.

    :return: the name of the county.

    """
    tag = node.find('h5', class_='obs-loc')
    location = tag.text.split('( Map )')[0]
    county = location.split(',')[-3].strip()
    return county


def _get_subnational1(node):
    """
    Get the region where the checklist was recorded.

    :param node: the Tag for the body of the Location section.

    :return: the name of the region.

    """
    tag = node.find('h5', class_='obs-loc')
    location = tag.text.split('( Map )')[0]
    region = location.split(',')[-2].strip()
    return region


def _get_country(node):
    """
    Get the country where the checklist was recorded.

    :param node: the Tag for the body of the Location section.

    :return: the two-letter ISO 8601 code doe the country.

    """
    tag = node.find('h5', class_='obs-loc')
    location = tag.text.split('( Map )')[0]
    country = location.split(',')[-1].strip()[:2]
    return country


def _get_location_identifier(node):
    """
    Get the unique identifier for the site if it is a hotspot.

    :param node: the Tag for the body of the Location section.

    :return: the unique identifier for the hotspot or None if it is not
    a hotspot.

    """
    identifier = None

    regex = re.compile(r'[Hh]otspot')
    tag = node.find('a', text=regex)

    if tag:
        url = tag['href']
        identifier = url.split('/')[-1]

    return identifier


def _get_latitude(node):
    """
    Get the latitude of the location.

    :param node: the Tag for the body of the Location section.

    :return: the signed latitude, positive values are north of the equator
    and negative ones are south of the equator.

    """
    tag = node.find('a', text='Map')
    url = tag['href']
    query = url.split('?')[1]
    params = query.split('&')
    coordinates = [param for param in params if param.startswith('ll=')][0]
    latitude = float(coordinates[3:].split(',')[0])
    return latitude


def _get_longitude(node):
    """
    Get the longitude of the location.

    :param node: the Tag for the body of the Location section.

    :return: the signed latitude, positive values are east of the prime
    meridian and negative ones are west of the prime meridian.

    """
    tag = node.find('a', text='Map')
    url = tag['href']
    query = url.split('?')[1]
    params = query.split('&')
    coordinates = [param for param in params if param.startswith('ll=')][0]
    longitude = float(coordinates[3:].split(',')[1])
    return longitude


def _point_protocol(node):
    """
    Get the effort for checklists following protocols where the observer(s)
    remain stationary.

    :param node: the tag for the body of the Date and Effort section.

    :return: the required and optional fields for the protocol.

    """
    results = {
        'time': _get_time(node),
        'duration': _get_duration(node),
        'party_size': _get_party_size(node),
        'observers': _get_observers(node),
    }

    if not results['time']:
        raise ValueError('the time field was not found')

    if not results['duration']:
        raise ValueError('the duration field was not found')

    if not results['party_size']:
        raise ValueError('the party size field was not found')

    return results


def _distance_protocol(node):
    """
    Get the effort for checklists following protocols where the observer(s)
    are moving.

    :param node: the tag for the body of the Date and Effort section.

    :return: the required and optional fields for the protocol.

    """
    results = {
        'time': _get_time(node),
        'duration': _get_duration(node),
        'distance': _get_distance(node),
        'party_size': _get_party_size(node),
        'observers': _get_observers(node),
    }

    if not results['time']:
        raise ValueError('the time field was not found')

    if results['duration'] is None:
        raise ValueError('the duration field was not found')

    if results['distance'] == (None, None):
        raise ValueError('the distance field was not found')

    if not results['party_size']:
        raise ValueError('the party size field was not found')

    return results


def _incidental_observations(node):
    """
    Get the effort for checklists with incidental observations.

    :param node: the tag for the body of the Date and Effort section.

    :return: the required and optional fields.

    """
    results = {
        'observers': _get_observers(node),
    }

    time = _get_time(node)
    if time:
        results['time'] = time

    return results


def _historical_observations(node):
    """
    Get the effort for checklists with historical records.

    All fields here are optional, except the observer submitting the
    checklist since any or no protocol may have been followed.

    :param node: the tag for the body of the Date and Effort section.

    :return: the required and optional fields for the protocol.

    """
    results = {
        'observers': _get_observers(node),
    }

    time = _get_time(node)
    if time:
        results['time'] = time

    duration = _get_duration(node)
    if duration:
        results['duration'] = duration

    distance = _get_distance(node)
    if distance != (None, None):
        results['distance'] = distance

    area = _get_area(node)
    if area != (None, None):
        results['area'] = area

    party_size = _get_party_size(node)
    if party_size:
        results['party_size'] = party_size

    return results


def _area_protocol(include_area=True):
    """
    Get the effort for checklists following protocols where the observer(s)
    are covering an area.

    A closure is used as the area field is optional for checklists that
    follow the Banding protocol so we avoid writing almost identical
    functions.

    :param include_area: the tag for the body of the Date and Effort section.

    :return: the required and optional fields for the protocol.

    """

    def _get_area_fields(node):
        results = {
            'time': _get_time(node),
            'area': _get_area(node),
            'duration': _get_duration(node),
            'party_size': _get_party_size(node),
            'observers': _get_observers(node),
        }

        if not results['time']:
            raise ValueError('the time field was not found')

        if include_area:
            if results['area'] == (None, None):
                raise ValueError('the area field was not found')
        else:
            del results['area']

        if results['duration'] is None:
            raise ValueError('the duration field was not found')

        if not results['party_size']:
            raise ValueError('the party size field was not found')

        return results

    return _get_area_fields


def _get_date_and_effort(node):
    """
    Get the date and details of the protocol, if any, used to create the checklist.

    :param node: the tag for the body of the Date and Effort section.

    :return: dict containing the checklist a
    """
    results = {
        'date': _get_date(node),
        'comment': _get_comment(node),
        'protocol': _get_protocol(node),
    }

    _get_effort = _get_date_and_effort.protocols[results['protocol']]
    results.update(_get_effort(node))

    return results


_get_date_and_effort.protocols = {
    'Stationary': _point_protocol,
    'Traveling': _distance_protocol,
    'Incidental': _incidental_observations,
    'Historical': _historical_observations,
    'Area': _area_protocol(),
    'Banding': _area_protocol(include_area=False),
    'eBird Pelagic Protocol': _distance_protocol,
    'Nocturnal Flight Call Count': _point_protocol,
    'Random': _distance_protocol,
    'CWC Point Count': _point_protocol,
    'CWC Area Count': _area_protocol(),
    'PROALAS': _point_protocol,
    'TNC California Waterbird Count': _point_protocol,
    'Rusty BlackbirdSpring Migration Blitz': _distance_protocol,
    'California Brown Pelican Survey': _distance_protocol,

}


def _get_protocol(node):
    """
    Get the method used to count the birds in the checklist.

    :param node: the Tag for the body of the Date and Effort section.

    :return: the name of the protocol used.

    """
    regex = re.compile(r'\s*[Pp]rotocol[:]?\s*')
    tag = node.find('dt', text=regex)
    field = tag.parent.dd
    protocol = field.text.strip().capitalize()
    return protocol


def _get_date(node):
    """
    Get the date the checklist was made.

    :param node: the Tag for the body of the Date and Effort section.

    :return: the date the checklist was recorded.

    """
    field = node.find('h5', class_='rep-obs-date')
    value = ' '.join(field.text.split()[:4])
    return datetime.datetime.strptime(value, '%a %b %d, %Y').date()


def _get_time(node):
    """
    Get the time the checklist was started.

    :param node: the Tag for the body of the Date and Effort section.

    :return: the time the checklist started. May be None for Incidental
    or Historical checklists.

    """
    time = None
    field = node.find('h5', class_='rep-obs-date')
    value = ' '.join(field.text.split()[4:])
    if value:
        time = datetime.datetime.strptime(value, '%I:%M %p').time()
    return time


def _get_party_size(node):
    """
    Get the number of observers.

    :param node: the Tag for the body of the Date and Effort section.

    :return: the number of people in the group.

    """
    count = None
    regex = re.compile(r'\s*[Pp]arty[\s]+[Ss]ize[:]?\s*')
    tag = node.find('dt', text=regex)
    if tag:
        field = tag.parent.dd
        count = int(field.text.strip())
    return count


def _get_distance(node):
    """
    Get the distance covered.

    :param node: the Tag for the body of the Date and Effort section.

    :return: a tuple containing the (floating-point) value and units for
    the distance covered, e.g (1.0, 'km') or (1.0, 'mi').

    """
    distance = None
    units = None

    regex = re.compile(r'\s*[Dd]istance[:]?\s*')
    tag = node.find('dt', text=regex)

    if tag:
        field = tag.parent.dd
        values = field.text.lower().split()
        distance = float(values[0])
        units = _get_distance.units[values[1]]

    return distance, units


_get_distance.units = {
    'kilometer(s)': 'km',
    'kilometre(s)': 'km',
    'km(s)':        'km',
    'kilometers':   'km',
    'kilometres':   'km',
    'km':           'km',
    'kms':          'km',
    'mile(s)':      'mi',
    'miles':        'mi',
}


def _get_area(node):
    """
    Get the area covered in hectares or acres.

    :param node: the Tag for the body of the Date and Effort section.

    :return: a tuple containing the (floating-point) value and units for
    the area covered, e.g (1.0, 'ha') or (1.0, 'acre').

    """
    area = None
    units = None

    regex = re.compile(r'\s*[Aa]rea[:]?\s*')
    tag = node.find('dt', text=regex)

    if tag:
        field = tag.parent.dd
        values = field.text.lower().split()
        area = float(values[0])
        units = _get_area.units[values[1]]

    return area, units


_get_area.units = {
    'hectare(s)': 'ha',
    'hectares':   'ha',
    'ha':         'ha',
    'acre(s)':    'acre',
    'acres':      'acre',
}


def _get_duration(node):
    """
    Get the time spent following the protocol.

    :param node: the Tag for the body of the Date and Effort section.

    :return: the time, in hours and minutes.

    """
    duration = None

    regex = re.compile(r'\s*[Dd]uration[:]?\s*')
    tag = node.find('dt', text=regex)

    if tag:
        field = tag.parent.dd
        values = field.text.lower().split()

        if 'minute(s)' in values:
            minutes = int(values[values.index('minute(s)') - 1])
        else:
            minutes = 0

        if 'hour(s)' in values:
            hours = int(values[values.index('hour(s)') - 1])
        else:
            hours = 0

        duration = datetime.timedelta(hours=hours, minutes=minutes)

    return duration


def _get_observers(node):
    """
    Get the list of observers.

    NOTE: this field only contains the name of the observer that submitted
    the checklist.

    :param node: the Tag for the body of the Date and Effort section.

    :return: the list of the observers' names.

    """
    observers = []

    regex = re.compile(r'\s*[Oo]bservers[:]?\s*')
    tag = node.find('dt', text=regex)
    field = tag.parent.dd
    name = ' '.join(field.text.split())
    observers.append(name)

    return observers


def _get_comment(node):
    """
    Get the comment about the checklist.

    :param node: the Tag for the body of the Date and Effort section.

    :return: the comment.

    """
    regex = re.compile(r'\s*[Cc]omment[:]?\s*')
    tag = node.find('dt', text=regex)

    comment = None

    if tag:
        field = tag.parent.dd
        comment = ' '.join(field.text.split()).capitalize()

    return comment


def _get_entries(node):
    """
    Get the list of species seen.
    
    :param node: the tag for body of the Species section.

    :return: a list of species seen.
    
    """
    entries = []
    tags = node.find_all('tr', class_='spp-entry')
    for tag in tags:
        entries.append(_get_entry(tag))
    return entries


def _get_entry(node):
    """
    Get the details of each species seen.

    :param node: the tag for an entry in the list of species.

    :return: a dict containg the fields extracted.

    """
    return {
        'species': _get_species(node),
        'count': _get_count(node),
    }


def _get_species(node):
    """
    Get common name of the species seen.

    :param node: the tag for an entry in the list of species.

    :return: the species common name.

    """
    tag = node.find('h5', class_='se-name')
    value = ' '.join(tag.text.split())
    return value


def _get_count(node):
    """
    Get the count for the species seen.

    :param node: the tag for an entry in the list of species.

    :return: the number of birds seen or None if the species was present
    but not counted.

    """
    count = None
    tag = node.find('h5', class_='se-count')
    value = tag.text.strip().lower()
    if value != 'x':
        count = int(value)
    return count


def _get_complete(node):
    """
    Get flag indicating whether the checklist contains all species seen.

    :param node: the tag for the body of the last section in the checklist.

    :return: True is all species seen were recorded, False if not.

    """
    tag = node.find('div', class_='all-spp-ans').find('h5')
    value = tag.text.strip().lower()
    return value == 'yes'



from bs4 import BeautifulSoup
from unittest import TestCase

from ebird.pages.checklists import _get_country


class ParseCountryTests(TestCase):
    """
    Tests for extracting the country code.

    """
    def setUp(self):
        self.template = """
        <div class="rs-body">
            <h5 class="bd obs-loc">%s</h5>
        <div>
        """
        self.value = None

    def parse(self):
        self.assertIsNotNone(self.value)
        content = self.template % self.value
        soup = BeautifulSoup(content, "lxml")
        tag = soup.find('div', class_='rs-body')
        return _get_country(tag)

    def test_country(self):
        """The country code is returned."""
        self.value = 'Site, Subnational2, Subnational1, CO'
        self.assertEqual('CO', self.parse())

    def test_extra_commas(self):
        """The country code is returned even though there are extra commas."""
        self.value = 'Site Name, With Commas, Subnational2, Subnational1, CO'
        self.assertEqual('CO', self.parse())

    def test_whitespace(self):
        """Whitespace trailing the country code is removed."""
        self.value = 'Site, Subnational2, Subnational1, CO \n'
        self.assertEqual('CO', self.parse())

    def test_period(self):
        """Any trailing period is removed from the country code."""
        self.value = 'Site, Subnational2, Subnational1, CO.'
        self.assertEqual('CO', self.parse())



from bs4 import BeautifulSoup
from unittest import TestCase

from ebird.pages.checklists import _get_entry


class GetEntryTests(TestCase):
    """
    Tests for each entry found in the checklist.

    """
    def setUp(self):
        self.content = None
        
    def parse(self):
        self.assertIsNotNone(self.content)
        soup = BeautifulSoup(self.content, "lxml")
        tag = soup.find('tr', class_='spp-entry')
        return _get_entry(tag)

    def test_correct_keys(self):
        """An entry contains the species and number seen."""
        self.content = """
            <tr class="spp-entry">
                <th><h5 class="se-count">3</h5></th>
                <td>
                    <h5 class="se-name">European Serin</h5>
                </td>
            </tr>
        """
        expected = ['count', 'species']
        self.assertListEqual(expected, sorted(self.parse().keys()))



from bs4 import BeautifulSoup
from datetime import date
from unittest import TestCase

from ebird.pages.checklists import _get_date


class GetDateTests(TestCase):
    """
    Tests for extracting the date and time a checklist was made.

    """
    def setUp(self):
        self.template = """
            <div class="rs-body-spp">
                <h5 class="rep-obs-date">%s</h5>
            </div>
        """
        self.value = None

    def parse(self):
        self.assertIsNotNone(self.value)
        content = self.template % self.value
        soup = BeautifulSoup(content, "lxml")
        tag = soup.find('div', class_='rs-body-spp')
        return _get_date(tag)

    def test_date(self):
        """The date is returned as a datetime.date object."""
        self.value = 'Fri Jul 28, 2017 3:15 PM'
        expected = date(2017, 7, 28)
        self.assertEqual(expected, self.parse())

    def test_extra_lines(self):
        """The date can be split across several lines."""
        self.value = 'Fri \n Jul 28,\n 2017 \n3:15 PM'
        expected = date(2017, 7, 28)
        self.assertEqual(expected, self.parse())



from bs4 import BeautifulSoup
from unittest import TestCase

from ebird.pages.checklists import _get_party_size


class FindPartySizeTests(TestCase):
    """
    Tests for finding the number of observers.

    """
    def setUp(self):
        self.template = """
            <div class="rs-body-spp">
            <dl class="def-list">
                <dt>%s</dt>
                <dd>3</dd>
            </dl>
            </div>
        """
        self.label = None
        self.expected = 3

    def parse(self):
        self.assertIsNotNone(self.label)
        content = self.template % self.label
        soup = BeautifulSoup(content, "lxml")
        tag = soup.find('div', class_='rs-body-spp')
        return _get_party_size(tag)

    def test_label(self):
        """The party size field can be found."""
        self.label = 'Party Size:'
        self.assertEqual(self.expected, self.parse())

    def test_label_multiple_lines(self):
        """The area label can be split over multiple lines."""
        self.label = '\nParty Size:\n'
        self.assertEqual(self.expected, self.parse())

    def test_label_lower_case(self):
        """The area label can be lower case."""
        self.label = 'party size:'
        self.assertEqual(self.expected, self.parse())

    def test_label_missing_colon(self):
        """The area label can have the trailing colon missing."""
        self.label = 'Party Size'
        self.assertEqual(self.expected, self.parse())


class ParsePartySizeTests(TestCase):
    """
    Tests for extracting the area covered in the checklist.

    """

    def setUp(self):
        self.template = """
            <div class="rs-body-spp">
            <dl class="def-list">
                <dt>Party Size:</dt>
                <dd>%s</dd>
            </dl>
            </div>
        """
        self.value = None

    def parse(self):
        self.assertIsNotNone(self.value)
        content = self.template % self.value
        soup = BeautifulSoup(content, "lxml")
        tag = soup.find('div', class_='rs-body-spp')
        return _get_party_size(tag)

    def test_integer(self):
        """The number of observers is returned as an integer."""
        self.value = '3'
        self.assertEqual(3, self.parse())

    def test_multiple_lines(self):
        """The number of observers can be split over multiple lines."""
        self.value = '\n 3 \n'
        self.assertEqual(3, self.parse())



"""
Command-line script to scape the web page(s) for a checklists and
save the data in JSON format to a file or to stdout.

"""

import click

from ebird.pages import get_checklist

from .base import save


# noinspection PyShadowingBuiltins
@click.command()
@click.option('--id', prompt=True,
              help="The unique identifier for the checklist.")
@click.option('--out', prompt=True, type=click.File('wb'),
              help='The name of a file to write the results to. To print'
                   ' the results to the screen use -.')
@click.option('--indent', type=int, default=None,
              help='Pretty-print the results with this level of indentation.')
def cli(id, out, indent):
    """Get the data for a checklist from its eBird web page."""
    save(out, get_checklist(id), indent)

if __name__ == '__main__':
    cli()



from datetime import time, timedelta

from bs4 import BeautifulSoup
from unittest import TestCase

from ebird.pages.checklists import _area_protocol


class AreaProtocoltTests(TestCase):
    """
    Tests for the fields extracted for checklist that followed the
    Area protocol.

    """
    def setUp(self):
        self.content = """
        <div class="report-section">
            <h5>Date and Effort</h5>
            <div class="rs-body-spp">
                <h5 class="rep-obs-date">Thu Aug 10, 2017 7:00 AM</h5>                                
            <dl class="def-list">
                <dt>Protocol:</dt>
                <dd>Area</dd>
            </dl>
            <dl class="def-list">
                <dt>Party Size:</dt>
                <dd>1</dd>
            </dl>
            <dl class="def-list">
                <dt>Area:</dt>
                <dd>4 hectares</dd>
            </dl>
            <dl class="def-list">
                <dt>Duration:</dt>
                <dd>4 hour(s)</dd>
            </dl>
            <dl class="def-list">
                <dt>Observers:</dt>
                <dd><strong>John Smith</strong></dd>
            </dl>
            <dl class="def-list report-comments">
                <dt>Comments:</dt>
                <dd></dd>
            </dl>
        </div>
        """

    def parse(self):
        soup = BeautifulSoup(self.content, "lxml")
        tag = soup.find('div', class_='report-section')
        return _area_protocol()(tag)

    def test_fields(self):
        """Only the fields for the Traveling protocol are returned."""
        expected = ['area', 'duration', 'observers', 'party_size', 'time']
        self.assertListEqual(expected, sorted(self.parse().keys()))

    def test_time(self):
        """The time is reported."""
        self.assertEqual(time(hour=7), self.parse()['time'])

    def test_area(self):
        """The area is reported."""
        self.assertEqual((4, 'ha'), self.parse()['area'])

    def test_duration(self):
        """The duration is reported."""
        self.assertEqual(timedelta(hours=4), self.parse()['duration'])

    def test_party_size(self):
        """The number of observers is reported."""
        self.assertEqual(1, self.parse()['party_size'])

    def test_observers(self):
        """The list of observers is reported."""
        self.assertEqual(['John Smith'], self.parse()['observers'])

    def test_time_missing(self):
        """An exception is raised if the start time is missing."""
        self.content = self.content.replace('7:00 AM', '')
        self.assertRaises(ValueError, self.parse)

    def test_area_missing(self):
        """An exception is raised if the area is missing."""
        self.content = self.content.replace('Area', '')
        self.assertRaises(ValueError, self.parse)

    def test_duration_missing(self):
        """An exception is raised if the duration is missing."""
        self.content = self.content.replace('Duration', '')
        self.assertRaises(ValueError, self.parse)

    def test_party_size_missing(self):
        """An exception is raised if the number of observers is missing."""
        self.content = self.content.replace('Party Size', '')
        self.assertRaises(ValueError, self.parse)

    def test_observers_missing(self):
        """An exception is raised if the number of observers is missing."""
        self.content = self.content.replace('Observers', '')
        self.assertRaises(AttributeError, self.parse)



from bs4 import BeautifulSoup
from unittest import TestCase

from ebird.pages.checklists import _get_protocol


class FindProtocolTests(TestCase):
    """
    Tests for finding the protocol used to compile the checklist.

    """
    def setUp(self):
        self.template = """
            <div class="rs-body-spp">
            <dl class="def-list">
                <dt>%s</dt>
                <dd>Stationary</dd>
            </dl>
            </div>
        """
        self.label = None
        self.expected = 'Stationary'

    def parse(self):
        self.assertIsNotNone(self.label)
        content = self.template % self.label
        soup = BeautifulSoup(content, "lxml")
        tag = soup.find('div', class_='rs-body-spp')
        return _get_protocol(tag)

    def test_label(self):
        """The protocol field can be found."""
        self.label = 'Protocol:'
        self.assertEqual(self.expected, self.parse())

    def test_label_multiple_lines(self):
        """The protocol label can be split over multiple lines."""
        self.label = '\nProtocol:\n'
        self.assertEqual(self.expected, self.parse())

    def test_label_lower_case(self):
        """The protocol label can be lower case."""
        self.label = 'protocol:'
        self.assertEqual(self.expected, self.parse())

    def test_label_missing_colon(self):
        """The protocol label can have the trailing colon missing."""
        self.label = 'Protocol'
        self.assertEqual(self.expected, self.parse())


class ParseProtocolTests(TestCase):
    """
    Tests for extracting the protocol used in the checklist.

    """
    def setUp(self):
        self.template = """
            <div class="rs-body-spp">
            <dl class="def-list">
                <dt>Protocol:</dt>
                <dd>%s</dd>
            </dl>
            </div>
        """
        self.value = None

    def parse(self):
        self.assertIsNotNone(self.value)
        content = self.template % self.value
        soup = BeautifulSoup(content, "lxml")
        tag = soup.find('div', class_='rs-body-spp')
        return _get_protocol(tag)

    def test_protocol(self):
        """The protocol is returned as a string."""
        self.value = 'Stationary'
        self.assertEqual('Stationary', self.parse())

    def test_value_multiple_lines(self):
        """The protocol may be split over multiple lines."""
        self.value = '\n Stationary \n'
        self.assertEqual('Stationary', self.parse())

    def test_value_lower_case(self):
        """The protocol may be lower case."""
        self.value = 'stationary'
        self.assertEqual('Stationary', self.parse())



from bs4 import BeautifulSoup
from unittest import TestCase

from ebird.pages.checklists import _get_subnational1


class GetSubnational1Tests(TestCase):
    """
    Tests for extracting the region (subnational1) from the checklist location.

    """
    def setUp(self):
        self.template = """
        <div class="rs-body">
            <h5 class="bd obs-loc">%s</h5>
        <div>
        """
        self.value = None

    def parse(self):
        self.assertIsNotNone(self.value)
        content = self.template % self.value
        soup = BeautifulSoup(content, "lxml")
        tag = soup.find('div', class_='rs-body')
        return _get_subnational1(tag)

    def test_region(self):
        """The name of the region is returned as a string."""
        self.value = 'Site, Subnational2, Subnational1, CO'
        self.assertEqual('Subnational1', self.parse())

    def test_with_hotspot_link(self):
        """The name of region is returned even if there is a link to the hotspot."""
        self.value = """
            Site, Subnational2, Subnational1, CO				
            <span class="btn-map">( <a href="/ebird/hotspot/L1234567">Hotspot</a> )</span>
        """
        self.assertEqual('Subnational1', self.parse())

    def test_commas(self):
        """Region is returned although sites have extra commas."""
        self.value = 'Site name, with comma, Subnational2, Subnational1, CO'
        self.assertEqual('Subnational1', self.parse())

    def test_extra_whitespace(self):
        """Extra whitespace is removed from the region."""
        self.value = 'Site, Subnational2,\n Subnational1 \n, CO'
        self.assertEqual('Subnational1', self.parse())



from bs4 import BeautifulSoup
from unittest import TestCase

from ebird.pages.checklists import _get_comment


class FindCommentTests(TestCase):
    """
    Tests for finding the checklist comment field.

    """
    def setUp(self):
        self.template = """
            <div class="rs-body-spp">
            <dl class="def-list">
                <dt>%s</dt>
                <dd>This is a comment.</dd>
            </dl>
            </div>
        """
        self.label = None
        self.expected = 'This is a comment.'

    def parse(self):
        self.assertIsNotNone(self.label)
        content = self.template % self.label
        soup = BeautifulSoup(content, "lxml")
        tag = soup.find('div', class_='rs-body-spp')
        return _get_comment(tag)

    def test_label(self):
        """The comment field can be found"""
        self.label = 'Comment:'
        self.assertEqual(self.expected, self.parse())

    def test_label_multiple_lines(self):
        """The comment label can be split over multiple lines."""
        self.label = '\nComment:\n'
        self.assertEqual(self.expected, self.parse())

    def test_label_lower_case(self):
        """The comment label can be lower case."""
        self.label = 'comment:'
        self.assertEqual(self.expected, self.parse())

    def test_label_missing_colon(self):
        """The comment label can have the trailing colon missing."""
        self.label = 'Comment'
        self.assertEqual(self.expected, self.parse())

    def test_comment_optional(self):
        """If the comment is missing then None is returned."""
        self.label = 'Other:'
        self.assertEqual(None, self.parse())


class GetCommentTests(TestCase):
    """
    Tests for extracting the checklist comment field.

    """
    def setUp(self):
        self.template = """
            <div class="rs-body-spp">
            <dl class="def-list">
                <dt>Comment:</dt>
                <dd>%s</dd>
            </dl>
            </div>
        """
        self.value = None

    def parse(self):
        self.assertIsNotNone(self.value)
        content = self.template % self.value
        soup = BeautifulSoup(content, "lxml")
        tag = soup.find('div', class_='rs-body-spp')
        return _get_comment(tag)

    def test_single_line(self):
        """Single line comments are extracted."""
        self.value = 'This is a comment.'
        self.assertEqual('This is a comment.', self.parse())

    def test_multiple_lines(self):
        """Comments with multiple lines are extracted."""
        self.value = 'This \n is \n a \n comment.'
        self.assertEqual('This is a comment.', self.parse())

    def test_whitespace(self):
        """Whitespace surrounding a comment in removed."""
        self.value = '\nThis is a comment.\n'
        self.assertEqual('This is a comment.', self.parse())

    def test_capitalized(self):
        """The first word in the comment is capitalized."""
        self.value = 'this is a comment.'
        self.assertEqual('This is a comment.', self.parse())



from datetime import timedelta

from bs4 import BeautifulSoup
from unittest import TestCase

from ebird.pages.checklists import _get_duration


class FindDurationTests(TestCase):
    """
    Tests for finding the time spent following the checklist protocol.

    """
    def setUp(self):
        self.template = """
            <div class="rs-body-spp">
            <dl class="def-list">
                <dt>%s</dt>
                <dd>30 minute(s)</dd>
            </dl>
            </div>
        """
        self.label = None
        self.expected = timedelta(minutes=30)

    def parse(self):
        self.assertIsNotNone(self.label)
        content = self.template % self.label
        soup = BeautifulSoup(content, "lxml")
        tag = soup.find('div', class_='rs-body-spp')
        return _get_duration(tag)

    def test_label(self):
        """The duration field can be found."""
        self.label = 'Duration:'
        self.assertEqual(self.expected, self.parse())

    def test_label_multiple_lines(self):
        """The duration label can be split over multiple lines."""
        self.label = '\n Duration: \n'
        self.assertEqual(self.expected, self.parse())

    def test_label_lower_case(self):
        """The duration label can be lower case."""
        self.label = 'duration:'
        self.assertEqual(self.expected, self.parse())

    def test_label_missing_colon(self):
        """The duration label can have the trailing colon missing."""
        self.label = 'Duration'
        self.assertEqual(self.expected, self.parse())

    def test_optional(self):
        """If the duration is cannot be found then None is returned."""
        self.label = 'Other:'
        self.expected = None
        self.assertEqual(self.expected, self.parse())


class ParseDurationTests(TestCase):
    """
    Tests for extracting the time spent following the checklist protocol.

    """
    def setUp(self):
        self.template = """
            <div class="rs-body-spp">
            <dl class="def-list">
                <dt>Duration:</dt>
                <dd>%s</dd>
            </dl>
            </div>
        """
        self.value = None

    def parse(self):
        self.assertIsNotNone(self.value)
        content = self.template % self.value
        soup = BeautifulSoup(content, "lxml")
        tag = soup.find('div', class_='rs-body-spp')
        return _get_duration(tag)

    def test_duration_hours(self):
        """The duration may be given in whole hours."""
        self.value = '1 hour(s)'
        self.assertEqual(timedelta(hours=1), self.parse())

    def test_duration_minutes(self):
        """The duration may be given in minutes."""
        self.value = '1 minute(s)'
        self.assertEqual(timedelta(minutes=1), self.parse())

    def test_duration_hours_and_minutes(self):
        """The duration may be given in hours and minutes."""
        self.value = '1 hour(s) 1 minute(s)'
        self.assertEqual(timedelta(hours=1, minutes=1), self.parse())

    def test_duration_multiple_lines(self):
        """The duration can be split over multiple lines."""
        self.value = '\n 30 \n minute(s) \n'
        self.assertEqual(timedelta(minutes=30), self.parse())



from datetime import time, timedelta

from bs4 import BeautifulSoup
from unittest import TestCase

from ebird.pages.checklists import _incidental_observations


class IncidentalProtocolTests(TestCase):
    """
    Tests for the fields extracted for checklists with incidental
    observations.

    """
    def setUp(self):
        self.content = """
        <div class="report-section">
            <h5>Date and Effort</h5>
            <div class="rs-body-spp">
                <h5 class="rep-obs-date">Thu Aug 10, 2017 7:00 AM</h5>                                
            <dl class="def-list">
                <dt>Protocol:</dt>
                <dd>Incidental</dd>
            </dl>
            <dl class="def-list">
                <dt>Observers:</dt>
                <dd><strong>John Smith</strong></dd>
            </dl>
            <dl class="def-list report-comments">
                <dt>Comments:</dt>
                <dd></dd>
            </dl>
        </div>
        """

    def parse(self):
        soup = BeautifulSoup(self.content, "lxml")
        tag = soup.find('div', class_='report-section')
        return _incidental_observations(tag)

    def test_fields(self):
        """Only the fields for the Traveling protocol are returned."""
        expected = ['observers', 'time']
        self.assertListEqual(expected, sorted(self.parse().keys()))

    def test_time(self):
        """The time is reported."""
        self.assertEqual(time(hour=7), self.parse()['time'])

    def test_observers(self):
        """The list of observers is reported."""
        self.assertEqual(['John Smith'], self.parse()['observers'])

    def test_time_missing(self):
        """No error is raised if the start time is missing."""
        self.content = self.content.replace('7:00 AM', '')
        self.assertFalse('time' in self.parse())

    def test_observers_missing(self):
        """An exception is raised if the number of observers is missing."""
        self.content = self.content.replace('Observers', '')
        self.assertRaises(AttributeError, self.parse)



from bs4 import BeautifulSoup
from unittest import TestCase

from ebird.pages.checklists import _get_latitude


class GetLatitudeTests(TestCase):
    """
    Tests for extracting the latitude of the location.

    """
    def setUp(self):
        self.template = """
        <div class="rs-body">
            <h5 class="bd obs-loc">	
                <span class="btn-map">( <a href="http://maps.com/?q=12.3456789,0.0&ll=%s,0.0">Map</a> )</span>
            </h5>
        <div>
        """
        self.value = None

    def parse(self):
        self.assertIsNotNone(self.value)
        content = self.template % self.value
        soup = BeautifulSoup(content, "lxml")
        tag = soup.find('div', class_='rs-body')
        return _get_latitude(tag)

    def test_latitude(self):
        """The latitude is returned as a float."""
        self.value = '12.3456789'
        self.assertEqual(12.3456789, self.parse())

    def test_negative(self):
        """Latitudes can be below the Equator are supported."""
        self.value = '-12.3456789'
        self.assertEqual(-12.3456789, self.parse())

    def test_query_parameter_order(self):
        """The latitude is returned even ff the query parameter order changes."""
        self.template = """
        <div class="rs-body">
            <h5 class="bd obs-loc">	
                <span class="btn-map">( <a href="http://maps.com/?ll=%s,0.0&q=-12.3456789,0.0">Map</a> )</span>
            </h5>
        <div>
        """
        self.value = '12.3456789'
        self.assertEqual(12.3456789, self.parse())

    def test_precision_changes(self):
        """The latitude may be presented in any precision."""
        self.value = '10.0'
        self.assertEqual(10.0, self.parse())



from bs4 import BeautifulSoup
from unittest import TestCase

from ebird.pages.checklists import _get_location_identifier


class GetLocationIdentifierTests(TestCase):
    """
    Tests for extracting the unique identifier of the location.

    """
    def setUp(self):
        self.template = """
        <div class="rs-body">
            <h5 class="bd obs-loc">%s</h5>
        <div>
        """
        self.tag = None

    def parse(self):
        self.assertIsNotNone(self.tag)
        content = self.template % self.tag
        soup = BeautifulSoup(content, "lxml")
        tag = soup.find('div', class_='rs-body')
        return _get_location_identifier(tag)

    def test_identifier(self):
        """If the location is a hotspot the uniwue identifier is returned."""
        self.tag = '<span class="btn-map">( <a href="/ebird/hotspot/L1234567">Hotspot</a> )</span>'
        self.assertEqual('L1234567', self.parse())

    def test_optional(self):
        """The identifier is None if the location is not a hotspot."""
        self.tag = ''
        self.assertEqual(None, self.parse())

    def test_extra_whitespace(self):
        """If the hotspot link text can contains whitespace."""
        self.tag = '<span class="btn-map">( <a href="/ebird/hotspot/L1234567">\n Hotspot \n</a> )</span>'
        self.assertEqual('L1234567', self.parse())

    def test_lower_case(self):
        """The hotspot link text can be lower case."""
        self.tag = '<span class="btn-map">( <a href="/ebird/hotspot/L1234567">hotspot</a> )</span>'
        self.assertEqual('L1234567', self.parse())



from bs4 import BeautifulSoup
from unittest import TestCase

from ebird.pages.checklists import _get_distance


class FindDistanceTests(TestCase):
    """
    Tests for finding the distance covered in the checklist.

    """
    def setUp(self):
        self.template = """
            <div class="rs-body-spp">
            <dl class="def-list">
                <dt>%s</dt>
                <dd>1.0 kilometer(s)</dd>
            </dl>
            </div>
        """
        self.label = None
        self.expected = (1.0, 'km')

    def parse(self):
        self.assertIsNotNone(self.label)
        content = self.template % self.label
        soup = BeautifulSoup(content, "lxml")
        tag = soup.find('div', class_='rs-body-spp')
        return _get_distance(tag)

    def test_label(self):
        """The distance field can be found."""
        self.label = 'Distance:'
        self.assertEqual(self.expected, self.parse())

    def test_label_multiple_lines(self):
        """The distance label can be split over multiple lines."""
        self.label = '\nDistance:\n'
        self.assertEqual(self.expected, self.parse())

    def test_label_lower_case(self):
        """The distance label can be lower case."""
        self.label = 'distance:'
        self.assertEqual(self.expected, self.parse())

    def test_label_missing_colon(self):
        """The distance label can have the trailing colon missing."""
        self.label = 'Distance'
        self.assertEqual(self.expected, self.parse())

    def test_distance_optional(self):
        """If the distance is missing a tuple containing None is returned."""
        self.label = 'Area:'
        self.assertEqual((None, None), self.parse())


class ParseDistanceTests(TestCase):
    """
    Tests for extracting the value distance covered in the checklist.

    """

    def setUp(self):
        self.template = """
            <div class="rs-body-spp">
            <dl class="def-list">
                <dt>Distance:</dt>
                <dd>%s</dd>
            </dl>
            </div>
        """
        self.value = None

    def parse(self):
        self.assertIsNotNone(self.value)
        content = self.template % self.value
        soup = BeautifulSoup(content, "lxml")
        tag = soup.find('div', class_='rs-body-spp')
        return _get_distance(tag)

    def test_kilometers(self):
        """The distance may be measured in kilometers."""
        self.value = '1.1 kilometer(s)'
        self.assertEqual((1.1, 'km'), self.parse())

    def test_miles(self):
        """The distance may be measured in miles."""
        self.value = '1.1 mile(s)'
        self.assertEqual((1.1, 'mi'), self.parse())

    def test_integer(self):
        """The distance is returned as a floating point number."""
        self.value = '1 kilometer(s)'
        self.assertEqual((1.0, 'km'), self.parse())

    def test_multiple_lines(self):
        """The distance can be split over multiple lines."""
        self.value = '\n 1.1 \n kilometer(s) \n'
        self.assertEqual((1.1, 'km'), self.parse())

    def test_text(self):
        """An exception is raised if the distance is not a number."""
        self.value = 'four kilometer(s)'
        self.assertRaises(ValueError, self.parse)

    def test_units_capitalized(self):
        """The distance units can be capitalized."""
        self.value = '1.1 Kilometer(s)'
        self.assertEqual((1.1, 'km'), self.parse())

    def test_units_abbreviated(self):
        """For a distance in kilometers, the units can be abbreviated to km."""
        self.value = '1.1 km(s)'
        self.assertEqual((1.1, 'km'), self.parse())

    def test_units_unknown(self):
        """An exception is raised if the distance units is not supported."""
        self.value = '1.1 furlongs'
        self.assertRaises(KeyError, self.parse)



from bs4 import BeautifulSoup
from unittest import TestCase

from ebird.pages.checklists import _get_area


class FindAreaTests(TestCase):
    """
    Tests for finding the area covered in the checklist.

    """
    def setUp(self):
        self.template = """
            <div class="rs-body-spp">
            <dl class="def-list">
                <dt>%s</dt>
                <dd>16 hectares</dd>
            </dl>
            </div>
        """
        self.label = None
        self.expected = (16, 'ha')

    def parse(self):
        self.assertIsNotNone(self.label)
        content = self.template % self.label
        soup = BeautifulSoup(content, "lxml")
        tag = soup.find('div', class_='rs-body-spp')
        return _get_area(tag)

    def test_label(self):
        """The area field can be found."""
        self.label = 'Area:'
        self.assertEqual(self.expected, self.parse())

    def test_label_multiple_lines(self):
        """The area label can be split over multiple lines."""
        self.label = '\nArea:\n'
        self.assertEqual(self.expected, self.parse())

    def test_label_lower_case(self):
        """The area label can be lower case."""
        self.label = 'area:'
        self.assertEqual(self.expected, self.parse())

    def test_label_missing_colon(self):
        """The area label can have the trailing colon missing."""
        self.label = 'Area'
        self.assertEqual(self.expected, self.parse())

    def test_area_optional(self):
        """If the area is missing a tuple containing None is returned."""
        self.label = 'Distance:'
        self.assertEqual((None, None), self.parse())


class ParseAreaTests(TestCase):
    """
    Tests for extracting the area covered in the checklist.

    """
    def setUp(self):
        self.template = """
            <div class="rs-body-spp">
            <dl class="def-list">
                <dt>Area:</dt>
                <dd>%s</dd>
            </dl>
            </div>
        """
        self.value = None

    def parse(self):
        self.assertIsNotNone(self.value)
        content = self.template % self.value
        soup = BeautifulSoup(content, "lxml")
        tag = soup.find('div', class_='rs-body-spp')
        return _get_area(tag)

    def test_hectares(self):
        """The area may be measured in hectares."""
        self.value = '1.1 hectares'
        self.assertEqual((1.1, 'ha'), self.parse())

    def test_acres(self):
        """The area may be measured in acres."""
        self.value = '1.1 acres'
        self.assertEqual((1.1, 'acre'), self.parse())

    def test_integer(self):
        """The area is returned as a floating point number."""
        self.value = '1 hectares'
        self.assertEqual((1.0, 'ha'), self.parse())

    def test_multiple_lines(self):
        """The area can be split over multiple lines."""
        self.value = '\n 1.1 \n hectares \n'
        self.assertEqual((1.1, 'ha'), self.parse())

    def test_text(self):
        """An exception is raised if the area is not a number."""
        self.value = 'four hectares'
        self.assertRaises(ValueError, self.parse)

    def test_units_capitalized(self):
        """The area units can be capitalized."""
        self.value = '1.1 Hectares'
        self.assertEqual((1.1, 'ha'), self.parse())

    def test_units_abbreviated(self):
        """The area in hectares can be abbreviated to ha."""
        self.value = '1.1 ha'
        self.assertEqual((1.1, 'ha'), self.parse())

    def test_units_unknown(self):
        """An exception is raised if the area units is not supported."""
        self.value = '1.1 sq. mile(s)'
        self.assertRaises(KeyError, self.parse)



from bs4 import BeautifulSoup
from unittest import TestCase

from ebird.pages.checklists import _get_complete


class ParseCompleteTests(TestCase):
    """
    Tests for extracting whether the checklist contains all species seen.

    """
    def setUp(self):
        self.template = """
            <div class="rs-body">
                <div class="all-spp-ans">
                    <h5>%s</h5>
                </div>
            </div>
        """
        self.value = None

    def parse(self):
        self.assertIsNotNone(self.value)
        content = self.template % self.value
        soup = BeautifulSoup(content, "lxml")
        tag = soup.find('div', class_='rs-body')
        return _get_complete(tag)

    def test_yes(self):
        """If all species were recorded, True is returned."""
        self.value = 'Yes'
        self.assertEqual(True, self.parse())

    def test_no(self):
        """If not all species were recorded, False is returned."""
        self.value = 'No'
        self.assertEqual(False, self.parse())

    def test_whitespace(self):
        """Whitespace surrounding all species recorded is removed."""
        self.value = '\nYes\n'
        self.assertEqual(True, self.parse())

    def test_lower_case(self):
        """All species removed can be in lower case"""
        self.value = 'yes'
        self.assertEqual(True, self.parse())



# -*- coding: utf-8 -*-

import os
import unittest

from setuptools import setup

from ebird.pages.version import __version__


def read(filename):
    with open(os.path.join(os.path.dirname(__file__), filename)) as fp:
        return fp.read()


def test_suite():
    # Force test discovery to only look at the tests directory
    # otherwise all the tests get executed twice.
    test_loader = unittest.TestLoader()
    return test_loader.discover('tests', pattern='test_*.py')


setup(
    name='ebird-pages',
    version=__version__,
    description='Scrapers for pages on the eBird web site',
    long_description=read("README.md"),
    author='ProjectBabbler',
    author_email='projectbabbler@gmail.com',
    url='http://pypi.python.org/pypi/ebird-pages/',
    license='GPL',
    keywords='eBird web scraper',
    packages=['ebird.pages'],
    test_suite='setup.test_suite',
    entry_points="""
        [console_scripts]
        get-checklist=ebird.pages.scripts.get_checklist:cli
    """,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Natural Language :: English',
        'Topic :: Utilities',
        'Topic :: Internet',
    ],
    install_requires=[
        'requests',
        'beautifulsoup4',
        'lxml',
        'Click'
    ],
)



from bs4 import BeautifulSoup
from unittest import TestCase

from ebird.pages.checklists import _get_species


class GetSpeciesTests(TestCase):
    """
    Tests for extracting the species from each entry in the list.

    """
    def setUp(self):
        self.template = """
            <tr class="spp-entry">
                <td><h5 class="se-name">%s</h5></td>
            </tr>
        """
        self.value = None

    def parse(self):
        self.assertIsNotNone(self.value)
        content = self.template % self.value
        soup = BeautifulSoup(content, "lxml")
        tag = soup.find('tr', class_='spp-entry')
        return _get_species(tag)

    def test_species(self):
        """The name of the species is returned as a string."""
        self.value = 'Barn Swallow'
        self.assertEqual('Barn Swallow', self.parse())

    def test_multiple_lines(self):
        """The species name can be split over multiple lines."""
        self.value = '\n Barn \n Swallow \n'
        self.assertEqual('Barn Swallow', self.parse())

    def test_subspecies(self):
        """The species name can have a subspecies designator."""
        self.value = 'Barn Swallow (White-bellied)'
        self.assertEqual('Barn Swallow (White-bellied)', self.parse())



from bs4 import BeautifulSoup
from unittest import TestCase

from ebird.pages.checklists import _get_location


class GetLocationTests(TestCase):
    """
    Tests for extracting the fields describing checklist location.

    """

    def setUp(self):
        self.content = None

    def get_keys(self):
        self.assertIsNotNone(self.content)
        soup = BeautifulSoup(self.content, "lxml")
        tag = soup.find('div', class_='rs-body')
        return sorted(_get_location(tag).keys())

    def test_correct_keys(self):
        """The location dict has the correct keys."""
        self.content = """
        <div class="rs-body">
            <h5 class="bd obs-loc">	
                Site, Subnational2, Subnational1, CO				
                <span class="btn-map">( <a href="http://maps.com/?q=1.0,2.0&ll=1.0,2.0">Map</a> )</span>
                <span class="btn-map">( <a href="/ebird/hotspot/L1234567">Hotspot</a> )</span>
            </h5>
        <div>
        """
        expected = ['country', 'identifier', 'lat', 'lon', 'name', 'subnational1', 'subnational2']
        self.assertListEqual(expected, self.get_keys())

    def test_optional_identifier(self):
        """The identifier is left out if the location is not a hotspot."""
        self.content = """
        <div class="rs-body">
            <h5 class="bd obs-loc">	
                Site, Subnational2, Subnational1, CO				
                <span class="btn-map">( <a href="http://maps.com/?&q=1.0,2.0&ll=1.0,2.0">Map</a> )</span>
            </h5>
        <div>
        """
        expected = ['country', 'lat', 'lon', 'name', 'subnational1', 'subnational2']
        self.assertListEqual(expected, self.get_keys())



import requests


def get_content(url: str) -> str:
    """
    Get the content from eBird web page.

    :param url: the URL for the API call.
    :type url: str

    :raises request.exceptions.ConnectionError if there is an error with
    the connection to the eBird site.

    :raises request.exceptions.Timeout if the eBird site takes too long
    to respond.

    :raises requests.exceptions.HTTPError any errors was returned by the
    eBird site.

    """
    response = requests.get(url)
    response.raise_for_status()
    return response.text


from bs4 import BeautifulSoup
from unittest import TestCase

from ebird.pages.checklists import _get_observers


class FindObserversTests(TestCase):
    """
    Tests for finding the list of observers.

    """
    def setUp(self):
        self.template = """
            <div class="rs-body-spp">
            <dl class="def-list">
                <dt>%s</dt>
                <dd><strong>John Smith</strong></dd>
            </dl>
            </div>
        """
        self.label = None
        self.expected = ['John Smith']

    def parse(self):
        self.assertIsNotNone(self.label)
        content = self.template % self.label
        soup = BeautifulSoup(content, "lxml")
        tag = soup.find('div', class_='rs-body-spp')
        return _get_observers(tag)

    def test_label(self):
        """The observers field can be found."""
        self.label = 'Observers:'
        self.assertEqual(self.expected, self.parse())

    def test_label_multiple_lines(self):
        """The observers label can be split over multiple lines."""
        self.label = '\nObservers:\n'
        self.assertEqual(self.expected, self.parse())

    def test_label_lower_case(self):
        """The observers label can be lower case."""
        self.label = 'observers:'
        self.assertEqual(self.expected, self.parse())

    def test_label_missing_colon(self):
        """The observers label can have the trailing colon missing."""
        self.label = 'Observers'
        self.assertEqual(self.expected, self.parse())

    def test_no_observers(self):
        """An exception is raised if no observers can be found"""
        self.label = 'Distance:'
        self.assertRaises(AttributeError, self.parse)


class ParseObserversTests(TestCase):
    """
    Tests for extracting the list of observers.

    """

    def setUp(self):
        self.template = """
            <div class="rs-body-spp">
            <dl class="def-list">
                <dt>Observers:</dt>
                <dd><strong>%s</strong></dd>
            </dl>
            </div>
        """
        self.value = None

    def parse(self):
        self.assertIsNotNone(self.value)
        content = self.template % self.value
        soup = BeautifulSoup(content, "lxml")
        tag = soup.find('div', class_='rs-body-spp')
        return _get_observers(tag)

    def test_observer(self):
        """The name of the observer who submitted the checklist is returned."""
        self.value = 'John Smith'
        self.assertListEqual(['John Smith'], self.parse())

    def test_multiple_lines(self):
        """The name of the observer can be split over multiple lines."""
        self.value = '\n John \n Smith \n'
        self.assertEqual(['John Smith'], self.parse())

    def test_not_strong(self):
        """The <strong> tag surrounding the observer name(s) may be omitted."""
        self.template = """
            <div class="rs-body-spp">
            <dl class="def-list">
                <dt>Observers:</dt>
                <dd>%s</dd>
            </dl>
            </div>
        """
        self.value = 'John Smith'
        self.assertEqual(['John Smith'], self.parse())



# -*- coding: utf-8 -*-

"""A set of functions for scraping data from eBird web pages."""

from .version import __version__

# Import all the functions that make up the public API.
# noinspection PyUnresolvedReferences
from ebird.pages.checklists import get_checklist

